import game_store from "../store/game";
import ble_store from "../store/bluetooth";
import Notify from "../../static/vant/notify/notify";

var $ff = "ff";
var $8f = "ffffffff"
export default {

  // 发送开启脑电数据包指令
  sendOpen() {
    this.WriteBufferInBle("03 00 00 00 ff");
  },
  // 发送开启教具脑控指令
  sendControl() {
    this.WriteBufferInBle("03 00 00 00 07");
  },
  //发送断开玩具的指令
  sendEnd() {
    //AA CC 03 00 00 00 09 F3
    this.WriteBufferInBle("03 00 00 00 09");
  },
  // 发送停止发送脑电数据包指令
  sendPause() {
    this.WriteBufferInBle("03 00 00 00 08");
  },

  //  获取大包数据 进行绘制图表
  get_big_data(hex) {
    if (hex.substr(0, 6) != "555520") {
      return false;
    }
    //当s1为 00时 数据有效
    let $s1 = hex.substr(8, 2);
    if ($s1 != "00") {
      return false;
    }

    //专注度数据
    let $att = parseInt("0x" + hex.substr(12, 2));
    //放松度数据
    let $med = parseInt("0x" + hex.substr(16, 2));

    //Delta数据
    let $delta_1 = hex.substr(hex.indexOf("0418") + 4, 2);
    let $delta_2 = hex.substr(hex.indexOf("0418") + 6, 2);
    let $delta_3 = hex.substr(hex.indexOf("0418") + 8, 2);
    let $delta = parseInt($delta_1 + $delta_2 + $delta_3, 16);

    //Theta数据
    let $theta_1 = hex.substr(hex.indexOf("0418") + 10, 2);
    let $theta_2 = hex.substr(hex.indexOf("0418") + 12, 2);
    let $theta_3 = hex.substr(hex.indexOf("0418") + 14, 2);
    let $theta = parseInt($theta_1 + $theta_2 + $theta_3, 16);


    //low_Alpha
    let $low_alpha_1 = hex.substr(hex.indexOf("0418") + 16, 2);
    let $low_alpha_2 = hex.substr(hex.indexOf("0418") + 18, 2);
    let $low_alpha_3 = hex.substr(hex.indexOf("0418") + 20, 2);
    let $low_alpha = parseInt($low_alpha_1 + $low_alpha_2 + $low_alpha_3, 16);

    //high_Alpha
    let $high_alpha_1 = hex.substr(hex.indexOf("0418") + 22, 2);
    let $high_alpha_2 = hex.substr(hex.indexOf("0418") + 24, 2);
    let $high_alpha_3 = hex.substr(hex.indexOf("0418") + 26, 2);
    let $high_alpha = parseInt($high_alpha_1 + $high_alpha_2 + $high_alpha_3, 16);

    //low_beta
    let $low_beta_1 = hex.substr(hex.indexOf("0418") + 28, 2);
    let $low_beta_2 = hex.substr(hex.indexOf("0418") + 30, 2);
    let $low_beta_3 = hex.substr(hex.indexOf("0418") + 32, 2);
    let $low_beta = parseInt($low_beta_1 + $low_beta_2 + $low_beta_3, 16);

    //high_beta
    let $high_beta_1 = hex.substr(hex.indexOf("0418") + 34, 2);
    let $high_beta_2 = hex.substr(hex.indexOf("0418") + 36, 2);
    let $high_beta_3 = hex.substr(hex.indexOf("0418") + 38, 2);
    let $high_beta = parseInt($high_beta_1 + $high_beta_2 + $high_beta_3, 16);
    console.log(
      "att: " + $att,
      "med: " + $med,
      "delta: " + $delta,
      "theta: " + $theta,
      "low_alpha: " + $low_alpha,
      "high_alpha: " + $high_alpha,
      "low_beta: " + $low_beta,
      "high_beta: " + $high_beta,
    );
    let $max_num = 30000;
    return {
      att: $att,
      med: $med,
      delta: $delta >= $max_num ? $max_num : $delta,
      theta: $theta >= $max_num ? $max_num : $theta,
      low_alpha: $low_alpha >= $max_num ? $max_num : $low_alpha,
      high_alpha: $high_alpha >= $max_num ? $max_num : $high_alpha,
      low_beta: $low_beta >= $max_num ? $max_num : $low_beta,
      high_beta: $high_beta >= $max_num ? $max_num : $high_beta
    };
  },
  //获取设备电量
  get_device_elc(hex) {
    if (hex.substr(0, 8) != "55550203") {
      return false;
    }
    let $power = parseInt("0x" + hex.substr(8, 2));
    return $power;
  },

  //  监听蓝牙连接状态
  watch_bluetooth_status($that) {
    let that = this;
    // 微信自身监听低功耗蓝牙连接状态的改变事件
    wx.onBLEConnectionStateChange((res) => {
      // 该方法回调中可以用于处理连接意外断开等异常情况
      ble_store.setters.setBluetoothLinkStatus(res.connected);
      if (res.connected == false) {
        //断开玩具连接
        try {
          $that.change_toy_connect_status(0);
          //断开蓝牙连接
          $that.change_device_status(0);
        } catch (e) {
          console.log("方法不存在");
        }

        //判断游戏是否游戏中
        let $game_status = game_store.getters.getGameStatus();
        if ($game_status == 1) {
          // $that.game_finished();
          //清除状态 设置为默认初始状态 0 小乌龟
          Notify({
            type: 'danger',
            duration: 0,
            message: '智脑环已断开连接，请重新扫码智脑环'
          });
          // game_store.setters.setGameStatus(0);
          // 结束状态更改为1
          game_store.setters.setGameCloseStatus(1);


        }
      }
    });
  },

  //获取蓝牙设备服务
  getBLEDeviceServices(deviceId) {
    const $this = this;

    wx.getBLEDeviceServices({
      deviceId,
      success: (res) => {
        for (let i = 0; i < res.services.length; i++) {
          console.log("serviceItem:" + res.services[i].uuid);
          if (res.services[i].uuid.indexOf('6E') != -1 || res.services[i].uuid.indexOf('0000FFF0') != -1) {
            console.log("SelectedServiceItem:" + res.services[i].uuid);
            $this.getBLEDeviceCharacteristics(deviceId, res.services[i].uuid);
            return;
          }
        }
      },
      fail(res) {
        console.log("连接蓝牙成功,获取服务失败");
      },
    });
  },

  //获取蓝牙设备某个服务中所有特征值
  getBLEDeviceCharacteristics(deviceId, serviceId) {
    console.log(deviceId, serviceId);
    const $this = this;
    wx.getBLEDeviceCharacteristics({
      deviceId,
      serviceId,
      success: (res) => {
        console.log("getBLEDeviceCharacteristics success", res.characteristics);
        for (let i = 0; i < res.characteristics.length; i++) {
          let item = res.characteristics[i];
          if (item.properties.read) {
            wx.readBLECharacteristicValue({
              deviceId,
              serviceId,
              characteristicId: item.uuid,
            });
          }
          if (item.properties.write) {
            $this.deviceId = deviceId;
            $this.serviceId = serviceId;
            $this.cid = item.uuid;
            game_store.setters.setDeviceId(deviceId);
            game_store.setters.setServiceId(serviceId);
            game_store.setters.setCid(item.uuid);
            //打开数据帧
            $this.sendOpen();
            // $this.sendPause(deviceId, serviceId, item.uuid)
            //获取玩具的值
            // bluetooth.sendConnect($this.toy_hex, deviceId, serviceId, item.uuid)
          }

          if (item.properties.notify || item.properties.indicate) {
            wx.notifyBLECharacteristicValueChange({
              deviceId,
              serviceId,
              characteristicId: item.uuid,
              state: true,
            });
          }
        }
      },
      fail(res) {
        console.error("getBLEDeviceCharacteristics", res);
      },
    });
  },


  // 发送连接玩具指令
  sendConnect($hex) {
    //获取最后两个字节
    this.WriteBufferInBle("03 00 " + $hex + " 00 0a");
  },
  //  获取当前连接得玩具
  getConnectedToy() {
    this.WriteBufferInBle("03 00 00 00 06");
  },
  // ArrayBuffer转16进度字符串示例
  ab2hex(buffer) {
    var hexArr = Array.prototype.map.call(
      new Uint8Array(buffer),
      function (bit) {
        return ("00" + bit.toString(16)).slice(-2);
      }
    );
    return hexArr.join("");
  },
  //断开发送数据
  shutdownSendControl() {
    this.WriteBufferInBle("03 00 00 00 0a");
  },

  // 监听脑环是否带正
  watchingDevice($this) {
    const that = this;
    wx.onBLECharacteristicValueChange((characteristic) => {
      let hexStr = that.ab2hex(characteristic.value);
      console.log(hexStr);
      let $data = that.get_big_data(hexStr);
      // wx.getBLEDeviceRSSI({
      //   deviceId: characteristic.deviceId,
      //   success(res) {
      //     $this.rssi = Math.abs(res["RSSI"]);
      //   },
      // });

      //监听连接教具
      if (hexStr.substr(0, 8).toLowerCase() == "aadd0a00") {
        if ($this.toy_item["hex"]) {
          let $hex = $this.toy_item["hex"].substr($this.toy_item["hex"].length - 2, 2);
          if (hexStr.substr(8, 2) == $hex) {
            wx.showToast({
              title: "已连接到" + $this.toy_item.name
            });
            $this.change_toy_connect_status(2);
          } else if (hexStr.substr(8, 2) == "00") {
            $this.change_toy_connect_status(3);
          }
        }
      }

      //暂停脑控，切断教具连接
      if (hexStr.substr(0, 8).toLowerCase() == "aadd0900") {
        this.shutdownSendControl();
      }


      //监听佩戴正确
      if (hexStr.substr(0, 6) == "555520") {
        //当s1为 00时 数据有效
        let $s1 = hexStr.substr(8, 2);
        // console.log("监听脑环是否带正：", $s1 == '00')

        $this.device_bg = $s1 == "00";
      }

      //  监听脑环电量
      let $power = that.get_device_elc(hexStr);
      if ($power) {
        // console.log("当前脑环电量:", $power)
        $this.device_power = $power;
      }
      if ($power < 10 && $power > 0) {
        wx.showToast({
          title: "脑环电量不足",
          icon: "none",
          duration: 2000,
          success() {
            // $this.change_device_status(0);
          },
        });
      }

      //游戏中模块
      let $game_status = game_store.getters.getGameStatus();
      if ($game_status == 1 && $data) {
        try {
          if ($this.played_time > 0) {
            //自定义定时器
            $this.played_time -= 1;
            game_store.setters.setPlayedTime($this.played_time)
            $this.played_time_text = that.formatPlaySeconds($this.played_time);
            $this.do_datas($data);
            if ($this.played_time_text.indexOf(":00") != -1 || $this.played_time_text.indexOf(":30") != -1) {
              $this.post_data();
            }
          }
          if ($this.played_time == 0) {
            $this.post_data();
            //判断是否隐藏 隐藏则不绘画
            that.shutdownSendControl();
            let $hide_status = game_store.getters.getHideStatus();
            if (!$hide_status) {
              $this.game_finished();
            }
          }
        } catch (e) {
          console.log("调用方法失败", e);
        }
      }
    });
  },
  // 游玩时间倒计时
  formatPlaySeconds(value) {
    // 字符串转数字
    var secondTime = parseInt(value); // 秒
    var minuteTime = 0; // 分
    if (secondTime > 60) {
      //如果秒数大于60，将秒数转换成整数
      //获取分钟，除以60取整数，得到整数分钟
      minuteTime = parseInt(secondTime / 60);
      //获取秒数，秒数取佘，得到整数秒数
      secondTime = parseInt(secondTime % 60);
      //如果分钟大于60，将分钟转换成小时
    }
    var result = "" + parseInt(secondTime);

    if (minuteTime > 0) {
      if (result.length == 1) {
        result = "0" + result;
      }
      if (parseInt(minuteTime).toString().length == 1) {
        minuteTime = "0" + parseInt(minuteTime);
      }
      result = "" + minuteTime + ":" + result;
    } else {
      result = "00:" + result;
    }
    return result;
  },

  //根据错误代码返回字符串信息
  GetopenBluetoothAdapterError(errCode) {

    let $errmsg = "";
    if (errCode == 10000) {
      $errmsg = "未初始化蓝牙适配器"
    }

    if (errCode == 10001) {
      $errmsg = "当前蓝牙适配器不可用"
    }

    if (errCode == 10002) {
      $errmsg = "没有找到指定设备"
    }

    if (errCode == 10003) {
      $errmsg = "连接失败"
    }

    if (errCode == 10006) {
      $errmsg = "当前连接已断开"
    }

    return $errmsg;
  },
  //写入buffer
  WriteBufferInBle($hex) {
    let $device_id = game_store.getters.getDeviceId();
    let $service_id = game_store.getters.getServiceId()
    let $charateristic = game_store.getters.getCharacterId();
    let $hex_header = "aa cc ";
    // let $hex = "03 00 01 00 0a";

    let $hex_sum = 0;
    let $hex_ary = $hex.split(" ");
    $hex_ary.forEach(($val, $index) => {
      $hex_sum += parseInt($val, 16);
    })

    let $checksum = ($hex_sum ^ parseInt($8f, 16)) & parseInt($ff, 16);

    $hex = $hex_header + $hex + " " + $checksum.toString(16);
    let buffer = new ArrayBuffer(8);
    let dataView = new DataView(buffer);
    $hex_ary = $hex.split(" ");
    $hex_ary.forEach(($val, $index) => {
      dataView.setUint8($index, parseInt($val, 16))
    })

    wx.writeBLECharacteristicValue({
      deviceId: $device_id,
      serviceId: $service_id,
      characteristicId: $charateristic,
      value: buffer,
      success: function (res) {
        console.log($hex + ',写入成功')
      },
      fail: function (err) {
        console.log($hex + "写入失败");
        console.log(err);
      },
    });
  }
};
